{-# OPTIONS_GHC -fno-warn-unused-imports    #-}
{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}

-- Derived from AWS service descriptions, licensed under Apache 2.0.

-- |
-- Module      : Network.AWS.CloudWatch
-- Copyright   : (c) 2013-2016 Brendan Hay
-- License     : Mozilla Public License, v. 2.0.
-- Maintainer  : Brendan Hay <brendan.g.hay@gmail.com>
-- Stability   : auto-generated
-- Portability : non-portable (GHC extensions)
--
-- Amazon CloudWatch monitors your Amazon Web Services (AWS) resources and
-- the applications you run on AWS in real-time. You can use CloudWatch to
-- collect and track metrics, which are the variables you want to measure
-- for your resources and applications.
--
-- CloudWatch alarms send notifications or automatically make changes to
-- the resources you are monitoring based on rules that you define. For
-- example, you can monitor the CPU usage and disk reads and writes of your
-- Amazon Elastic Compute Cloud (Amazon EC2) instances and then use this
-- data to determine whether you should launch additional instances to
-- handle increased load. You can also use this data to stop under-used
-- instances to save money.
--
-- In addition to monitoring the built-in metrics that come with AWS, you
-- can monitor your own custom metrics. With CloudWatch, you gain
-- system-wide visibility into resource utilization, application
-- performance, and operational health.
module Network.AWS.CloudWatch
    (
    -- * Service Configuration
      cloudWatch

    -- * Errors
    -- $errors

    -- ** LimitExceededFault
    , _LimitExceededFault

    -- ** InvalidNextToken
    , _InvalidNextToken

    -- ** InternalServiceFault
    , _InternalServiceFault

    -- ** InvalidParameterValueException
    , _InvalidParameterValueException

    -- ** InvalidFormatFault
    , _InvalidFormatFault

    -- ** MissingRequiredParameterException
    , _MissingRequiredParameterException

    -- ** InvalidParameterCombinationException
    , _InvalidParameterCombinationException

    -- ** ResourceNotFound
    , _ResourceNotFound

    -- * Waiters
    -- $waiters

    -- * Operations
    -- $operations

    -- ** EnableAlarmActions
    , module Network.AWS.CloudWatch.EnableAlarmActions

    -- ** PutMetricData
    , module Network.AWS.CloudWatch.PutMetricData

    -- ** DescribeAlarms (Paginated)
    , module Network.AWS.CloudWatch.DescribeAlarms

    -- ** ListMetrics (Paginated)
    , module Network.AWS.CloudWatch.ListMetrics

    -- ** DeleteAlarms
    , module Network.AWS.CloudWatch.DeleteAlarms

    -- ** DescribeAlarmHistory (Paginated)
    , module Network.AWS.CloudWatch.DescribeAlarmHistory

    -- ** GetMetricStatistics
    , module Network.AWS.CloudWatch.GetMetricStatistics

    -- ** DescribeAlarmsForMetric
    , module Network.AWS.CloudWatch.DescribeAlarmsForMetric

    -- ** DisableAlarmActions
    , module Network.AWS.CloudWatch.DisableAlarmActions

    -- ** PutMetricAlarm
    , module Network.AWS.CloudWatch.PutMetricAlarm

    -- ** SetAlarmState
    , module Network.AWS.CloudWatch.SetAlarmState

    -- * Types

    -- ** ComparisonOperator
    , ComparisonOperator (..)

    -- ** HistoryItemType
    , HistoryItemType (..)

    -- ** StandardUnit
    , StandardUnit (..)

    -- ** StateValue
    , StateValue (..)

    -- ** Statistic
    , Statistic (..)

    -- ** AlarmHistoryItem
    , AlarmHistoryItem
    , alarmHistoryItem
    , ahiAlarmName
    , ahiHistoryItemType
    , ahiHistoryData
    , ahiHistorySummary
    , ahiTimestamp

    -- ** Datapoint
    , Datapoint
    , datapoint
    , dSampleCount
    , dMaximum
    , dAverage
    , dMinimum
    , dSum
    , dUnit
    , dTimestamp

    -- ** Dimension
    , Dimension
    , dimension
    , dName
    , dValue

    -- ** DimensionFilter
    , DimensionFilter
    , dimensionFilter
    , dfValue
    , dfName

    -- ** Metric
    , Metric
    , metric
    , mMetricName
    , mNamespace
    , mDimensions

    -- ** MetricAlarm
    , MetricAlarm
    , metricAlarm
    , maAlarmName
    , maStateUpdatedTimestamp
    , maPeriod
    , maAlarmDescription
    , maEvaluationPeriods
    , maMetricName
    , maNamespace
    , maComparisonOperator
    , maOKActions
    , maStateValue
    , maThreshold
    , maAlarmConfigurationUpdatedTimestamp
    , maActionsEnabled
    , maInsufficientDataActions
    , maStateReason
    , maStateReasonData
    , maDimensions
    , maAlarmARN
    , maAlarmActions
    , maUnit
    , maStatistic

    -- ** MetricDatum
    , MetricDatum
    , metricDatum
    , mdValue
    , mdDimensions
    , mdUnit
    , mdTimestamp
    , mdStatisticValues
    , mdMetricName

    -- ** StatisticSet
    , StatisticSet
    , statisticSet
    , ssSampleCount
    , ssSum
    , ssMinimum
    , ssMaximum
    ) where

import           Network.AWS.CloudWatch.DeleteAlarms
import           Network.AWS.CloudWatch.DescribeAlarmHistory
import           Network.AWS.CloudWatch.DescribeAlarms
import           Network.AWS.CloudWatch.DescribeAlarmsForMetric
import           Network.AWS.CloudWatch.DisableAlarmActions
import           Network.AWS.CloudWatch.EnableAlarmActions
import           Network.AWS.CloudWatch.GetMetricStatistics
import           Network.AWS.CloudWatch.ListMetrics
import           Network.AWS.CloudWatch.PutMetricAlarm
import           Network.AWS.CloudWatch.PutMetricData
import           Network.AWS.CloudWatch.SetAlarmState
import           Network.AWS.CloudWatch.Types
import           Network.AWS.CloudWatch.Waiters

{- $errors
Error matchers are designed for use with the functions provided by
<http://hackage.haskell.org/package/lens/docs/Control-Exception-Lens.html Control.Exception.Lens>.
This allows catching (and rethrowing) service specific errors returned
by 'CloudWatch'.
-}

{- $operations
Some AWS operations return results that are incomplete and require subsequent
requests in order to obtain the entire result set. The process of sending
subsequent requests to continue where a previous request left off is called
pagination. For example, the 'ListObjects' operation of Amazon S3 returns up to
1000 objects at a time, and you must send subsequent requests with the
appropriate Marker in order to retrieve the next page of results.

Operations that have an 'AWSPager' instance can transparently perform subsequent
requests, correctly setting Markers and other request facets to iterate through
the entire result set of a truncated API operation. Operations which support
this have an additional note in the documentation.

Many operations have the ability to filter results on the server side. See the
individual operation parameters for details.
-}

{- $waiters
Waiters poll by repeatedly sending a request until some remote success condition
configured by the 'Wait' specification is fulfilled. The 'Wait' specification
determines how many attempts should be made, in addition to delay and retry strategies.
-}
